package kagent.test;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Properties;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;

import kagent.test.changeset.Add;
import kagent.test.changeset.ChangeSet;
import kagent.test.changeset.ObjectFactory;
import kagent.test.changeset.Remove;

import org.apache.log4j.Logger;
import org.drools.KnowledgeBase;
import org.drools.KnowledgeBaseConfiguration;
import org.drools.KnowledgeBaseFactory;
import org.drools.RuleBase;
import org.drools.SessionConfiguration;
import org.drools.agent.KnowledgeAgent;
import org.drools.agent.KnowledgeAgentFactory;
import org.drools.builder.KnowledgeBuilder;
import org.drools.common.DroolsObjectInputStream;
import org.drools.common.DroolsObjectOutputStream;
import org.drools.common.InternalRuleBase;
import org.drools.definition.type.FactType;
import org.drools.impl.KnowledgeBaseImpl;
import org.drools.impl.StatefulKnowledgeSessionImpl;
import org.drools.io.Resource;
import org.drools.marshalling.Marshaller;
import org.drools.marshalling.MarshallerFactory;
import org.drools.marshalling.ObjectMarshallingStrategy;
import org.drools.marshalling.ObjectMarshallingStrategyAcceptor;
import org.drools.reteoo.ReteooStatefulSession;
import org.drools.rule.Package;
import org.drools.runtime.Environment;
import org.drools.runtime.StatefulKnowledgeSession;

public abstract class AbstractDroolsEngine implements DroolsEngineService {

	protected StatefulKnowledgeSession ksession;
	protected KnowledgeAgent kagent;
	protected static Logger log = Logger
			.getLogger(AbstractDroolsEngine.class);

	protected static String changesetPath;
	protected static Resource engineResource;

	private ReteooStatefulSession wm;

	public AbstractDroolsEngine(FileInputStream input)
			throws IOException, ClassNotFoundException {
		loadCorrelator(input);
	}

	public AbstractDroolsEngine(KnowledgeBuilder kbuilder,
			Object[] globals, Environment env,
			KnowledgeBaseConfiguration config, SessionConfiguration sessionConf) {

		this(kbuilder, globals, env, config, sessionConf, null,
				false);
	}

	public AbstractDroolsEngine(KnowledgeBuilder kbuilder,
			Object[] globals, Environment env,
			KnowledgeBaseConfiguration config,
			SessionConfiguration sessionConf, String changesetPath) {

		this(kbuilder, globals, env, config, sessionConf,
				changesetPath, true);
	}

	public AbstractDroolsEngine(KnowledgeBuilder kbuilder,
			Object[] globals, Environment env,
			KnowledgeBaseConfiguration config,
			SessionConfiguration sessionConf, String changesetPath,
			Boolean hasKnowledgeAgent) {		

		KnowledgeBase kbase = createKBase(kbuilder, config);

		if (hasKnowledgeAgent) {
			createKnowledgeAgent(kbase, "MyKAgent");
			wm = (ReteooStatefulSession) ((KnowledgeBaseImpl) kagent
					.getKnowledgeBase()).ruleBase.newStatefulSession(
					(SessionConfiguration) sessionConf, env);

			ksession = new StatefulKnowledgeSessionImpl(wm, kagent
					.getKnowledgeBase());
		} else {
			ksession = kbase.newStatefulKnowledgeSession(sessionConf, env);
		}
		for (Object global : globals) {
			if (ksession.getGlobal(global.getClass().getName()) != null)
				ksession.setGlobal(global.getClass().getName(), global);
		}
	}

	/**
	 * Creates KnowledgeBase from KBuilder and KBaseConfig
	 * 
	 * @param kbuilder
	 * @param config
	 * @return
	 */
	protected KnowledgeBase createKBase(KnowledgeBuilder kbuilder,
			KnowledgeBaseConfiguration config) {
		KnowledgeBase kbase = KnowledgeBaseFactory.newKnowledgeBase(config);
		kbase.addKnowledgePackages(kbuilder.getKnowledgePackages());
		return kbase;
	}

	/**
	 * Creates a KnowledgeAgent for loading the RuleBase changes dynamically
	 * 
	 * @param kbase
	 * @param correlatorId
	 */
	protected void createKnowledgeAgent(KnowledgeBase kbase, String engineName) {
		Properties agentProps = new Properties();
		agentProps.setProperty("drools.agent.scanResources", "true");
		agentProps.setProperty("drools.agent.monitorChangeSetEvents", "true");
		agentProps.setProperty("drools.agent.scanDirectories", "true");

		kagent = KnowledgeAgentFactory.newKnowledgeAgent(engineName, kbase,
				KnowledgeAgentFactory
						.newKnowledgeAgentConfiguration(agentProps));
	}

	/**
	 * Updates the KnowledgeSession with the KnowledgeAgent RuleBase info
	 */
	public void updateKSession() {
		if (kagent != null) {
			System.out.println("*************** Updating KSession");
			
			wm.setRuleBase((InternalRuleBase) ((KnowledgeBaseImpl) kagent
					.getKnowledgeBase()).ruleBase);
			
			ksession = new StatefulKnowledgeSessionImpl(wm, kagent
					.getKnowledgeBase());
			
			System.out.println("*************** #Rule:" + ksession.getKnowledgeBase().getKnowledgePackages().iterator().next().getRules().size());
		}
	}

	public void saveCorrelator(FileOutputStream out) throws IOException {

		DroolsObjectOutputStream droolsOut = new DroolsObjectOutputStream(out);
		droolsOut.writeObject(ksession.getKnowledgeBase());
		Marshaller mas = createMarshaller(ksession.getKnowledgeBase());
		mas.marshall(droolsOut, ksession);
	}

	public void loadCorrelator(FileInputStream input) throws IOException,
			ClassNotFoundException {

		DroolsObjectInputStream droolsIn = new DroolsObjectInputStream(input);
		KnowledgeBase kbase = (KnowledgeBase) droolsIn.readObject();
		Marshaller mas = createMarshaller(kbase);
		ksession = mas.unmarshall(droolsIn);
	}

	private Marshaller createMarshaller(KnowledgeBase kbase) {
		ObjectMarshallingStrategyAcceptor acceptor = MarshallerFactory
				.newClassFilterAcceptor(new String[] { "*.*" });
		ObjectMarshallingStrategy strategy = MarshallerFactory
				.newSerializeMarshallingStrategy(acceptor);
		return MarshallerFactory.newMarshaller(kbase,
				new ObjectMarshallingStrategy[] { strategy });
	}

	protected static void createChangeSetInfo(String rulePackagePath,
			String droolsEngineName, Boolean add) {

		ObjectFactory of = new ObjectFactory();

		ChangeSet cs = of.createChangeSet();
		kagent.test.changeset.Resource rs = of.createResource();
		rs.setSource(rulePackagePath);
		rs.setType("DRL");

		if (add) {
			Add ad = of.createAdd();
			ad.getContent().add(rs);
			cs.setAdd(ad);
		} else {
			Remove rm = of.createRemove();
			rm.getContent().add(rs);
			cs.setRemove(rm);
		}

		JAXBContext jaxbContext;
		try {
			jaxbContext = JAXBContext.newInstance(ChangeSet.class.getPackage()
					.getName());
			javax.xml.bind.Marshaller marshaller = jaxbContext
					.createMarshaller();

			File file = new File(changesetPath + droolsEngineName + ".xml");
			if (file.exists())
				file.delete();
			marshaller.marshal(cs, new FileWriter(changesetPath + droolsEngineName
					+ ".xml"));
		} catch (JAXBException e) {
			log.error(e);
		} catch (FileNotFoundException e) {
			log.error(e);
		} catch (IOException e) {
			log.error(e);
		}
	}
	
	public abstract void insertFact(Object fact);
}
