package kagent.test;

import java.io.FileInputStream;
import java.io.IOException;
import java.util.Iterator;

import org.drools.ClockType;
import org.drools.KnowledgeBaseConfiguration;
import org.drools.KnowledgeBaseFactory;
import org.drools.SessionConfiguration;
import org.drools.builder.KnowledgeBuilder;
import org.drools.builder.KnowledgeBuilderError;
import org.drools.builder.KnowledgeBuilderFactory;
import org.drools.builder.ResourceType;
import org.drools.conf.EventProcessingOption;
import org.drools.io.ResourceFactory;
import org.drools.runtime.Environment;

public class TemporalDroolsEngineServiceImpl extends AbstractDroolsEngine {

	private String rulePackagePath;

	private static KnowledgeBuilder getKnowledgeBuilder(String rulePackagePath, String droolsEngineName) {
		KnowledgeBuilder kbuilder = KnowledgeBuilderFactory
				.newKnowledgeBuilder();
		
		changesetPath = "var/";
		
		createChangeSetInfo(rulePackagePath, droolsEngineName, true);
		engineResource = ResourceFactory.newFileResource(changesetPath
				+ droolsEngineName + ".xml");
		kbuilder.add(engineResource, ResourceType.CHANGE_SET);

		if (kbuilder.hasErrors()) {
			Iterator<KnowledgeBuilderError> ite = kbuilder.getErrors()
					.iterator();
			while (ite.hasNext()) {
				System.out.println(ite.next().getMessage());
			}
			return null;
		} else {
			return kbuilder;
		}
	}

	private static KnowledgeBaseConfiguration getKBaseConfig() {
		KnowledgeBaseConfiguration config = KnowledgeBaseFactory
				.newKnowledgeBaseConfiguration();
		config.setOption(EventProcessingOption.STREAM);
		return config;
	}

	private static SessionConfiguration getSessionConf() {
		SessionConfiguration sessionConf = new SessionConfiguration();
		sessionConf.setClockType(ClockType.REALTIME_CLOCK);
		return sessionConf;
	}

	public TemporalDroolsEngineServiceImpl(FileInputStream input,String engineName) throws IOException, ClassNotFoundException {

		super(input);
		
		createKnowledgeAgent(ksession.getKnowledgeBase(), engineName);		
		kagent.applyChangeSet(ResourceFactory.newFileResource(changesetPath
				+ engineName + ".xml"));
	}

	public TemporalDroolsEngineServiceImpl(String rulePackagePath,
			String engineName, Object[] globals, Environment env,
			String changesetPath) {

		super(getKnowledgeBuilder(rulePackagePath,
				engineName), globals, env, getKBaseConfig(),
				getSessionConf(), changesetPath);

		kagent.applyChangeSet(engineResource);
		this.rulePackagePath = rulePackagePath;
	}

	public void insertFact(Object fact) {
		ksession.getWorkingMemoryEntryPoint("MyEntryPoint").insert(fact);
		ksession.fireAllRules();
	}

	public void removeCorrelator() {
	}
}
